(defconst carbon-fontspec-regexp nil)

(let ((name-regexp "\\(\\([^\\:]\\|\\\\\\\\\\|\\\\:\\)+\\)")
      (size-regexp ":\\([0-9]+\\)")
      (style-regexp ":\\(b\\|i\\|bi\\|bold\\|italic\\|bolditalic\\)"))
  (setq
   carbon-fontspec-regexp
   (concat
    "^" name-regexp "\\(" size-regexp "\\(" style-regexp "\\)?" "\\)?" "$")))

(defun carbon-init-global-faces ()
  (or (face-foreground 'default 'global)
      (set-face-foreground 'default "wheat" 'global '(carbon default)))
  (or (face-background 'default 'global)
      (set-face-background 'default "darkslategrey" 'global '(carbon default))))

(defun carbon-init-device-faces (device)
  )

(defun carbon-init-frame-faces (frame)
  )

(defun carbon-parse-fontspec (fontspec)
  (cond ((string-match carbon-fontspec-regexp fontspec)
	 (list (match-string 1 fontspec) 
	       (let ((size (match-string 4 fontspec)))
		 (cond (size size)
		       (t "10")))
	       (let ((style (match-string 6 fontspec)))
		 (cond (style style)
		       (t "")))))
	(t nil)))

(defun carbon-font-name (parsed-fontspec)
  (car parsed-fontspec))

(defun carbon-font-size (parsed-fontspec)
  (cadr parsed-fontspec))

(defun carbon-font-style (parsed-fontspec)
  (caddr parsed-fontspec))

(defun carbon-make-font-bold (font &optional device)
  "Given a Carbon font specification, this attempts to make a bold font.
If it fails, it returns nil."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (cond ((or 
		   (string-equal (carbon-font-style parsed-fontspec) "i")
		   (string-equal (carbon-font-style parsed-fontspec) "bi"))
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":bi"))
		 (t
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":b"))))
	  (t nil))))

(defun carbon-make-font-italic (font &optional device)
  "Given a Carbon font specification, this attempts to make an `italic'
font. If it fails, it returns nil."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (cond ((or 
		   (string-equal (carbon-font-style parsed-fontspec) "b")
		   (string-equal (carbon-font-style parsed-fontspec) "bi"))
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":bi"))
		 (t
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":i"))))
	  (t nil))))

(defun carbon-make-font-bold-italic (font &optional device)
  "Given a Carbon font specification, this attempts to make a `bold-italic'
font. If it fails, it returns nil."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (concat (carbon-font-name parsed-fontspec)
		   ":"
		   (carbon-font-size parsed-fontspec)
		   ":bi"))
	  (t nil))))

(defun carbon-make-font-unbold (font &optional device)
  "Given a Carbon font specification, this attempts to make a non-bold font.
If it fails, it returns nil."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (cond ((string-equal (carbon-font-style parsed-fontspec) "b")
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)))
		 ((string-equal (carbon-font-style parsed-fontspec) "bi")
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":i"))
		 (t font)))
	  (t nil))))

(defun carbon-make-font-unitalic (font &optional device)
  "Given a Carbon font specification, this attempts to make a non-italic
font. If it fails, it returns nil."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (cond ((string-equal (carbon-font-style parsed-fontspec) "i")
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)))
		 ((string-equal (carbon-font-style parsed-fontspec) "bi")
		  (concat (carbon-font-name parsed-fontspec)
			  ":"
			  (carbon-font-size parsed-fontspec)
			  ":b"))
		 (t font)))
	  (t nil))))

(defun carbon-find-smaller-font (font &optional device)
  "Loads a new version of the given font (or font name) 1 point smaller.
Returns the font if it succeeds, nil otherwise."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (concat (carbon-font-name parsed-fontspec)
		   ":"
		   (number-to-string
		    (1- (string-to-number
			 (carbon-font-size parsed-fontspec))))
		   (cond ((string-equal (carbon-font-style parsed-fontspec) "")
			  "")
			 (t
			  (concat ":" (carbon-font-style parsed-fontspec))))))
	  (t nil))))

(defun carbon-find-larger-font (font &optional device)
  "Loads a new version of the given font (or font name) 1 point larger.
Returns the font if it succeeds, nil otherwise."
  (let ((parsed-fontspec (carbon-parse-fontspec font)))
    (cond (parsed-fontspec
	   (concat (carbon-font-name parsed-fontspec)
		   ":"
		   (number-to-string
		    (1+ (string-to-number
			 (carbon-font-size parsed-fontspec))))
		   (cond ((string-equal (carbon-font-style parsed-fontspec) "")
			  "")
			 (t
			  (concat ":" (carbon-font-style parsed-fontspec))))))
	  (t nil))))

